/*

Directory Wide Structure Comparison

Problem: You have clients that have been using your software, and you
have been developing the program. Now it is time to give the client the
new software, but you must insure that the client data files match the
structure that is expected by the new program.

Solution: Create a utility that compares a known set of data structures
with the ones the client is currently using, to inform you which ones
need to be updated, and why.

To use, place a set of the most current data files on a floppy disk,
along with this program. The data files on the floppy can be empty, but
must be the structures that contain the correct structures for the
current software.

Change directory to the working data set and type a:strucomp<enter>

This program will check the current directory against an empty data set
to see if there have been any structural changes, or if the file is
still needed at all. It will also check to see if there are new files
that need to be added to the client directory.

*/

#include "common.ch"

function strucomp( otherdir )

local s, w, x, y, z, newlist, clientlist, files

default otherdir to ''

if '/?' $ otherdir .or. empty( otherdir )
   ? 'Usage:'
   ? '     STRUCOMP [somepath]'
   ?
   ? '     somepath defaults to A:\ if not specified.'
   ?
   quit
endif

default otherdir to 'A:\'

files := directory('*.dbf')

for x := 1 to len(files)
   dbcloseall()
   use (files[x,1]) new alias clientdata

   // check to see if the file we are checking is still an active file.

   if file( otherdir+files[x,1] )
      use (otherdir+files[x,1]) new alias newdata
   else
      ? 'File '+files[x,1]+' is no longer needed'
      loop
   endif

   // then check field count in both tables

   select newdata
   newlist := dbstruct()
   select clientdata
   clientlist := dbstruct()

   y := len( newlist )
   z := len( clientlist )

   if y <> z
      ? 'File '+files[x,1]+' has wrong number of fields'
      loop
   endif

   /*
   if you get this far, check for structural differences this checks
   ordinal position, field name, type, length and decimal any
   differences will cause the check to end on the current file.
   */

   begin sequence
   for w := 1 to y
      for s := 1 to 4
         if newlist[w,s] <> clientlist[w,s]
            ? 'File '+files[x,1]+' has structure change'
            break
         endif
      next
   next
   end sequence

next

dbcloseall()

/*
This loop checks to see if you have any
new files that the client does not have
*/

files := directory(otherdir+'*.dbf')

for x := 1 to len(files)
   if !file( files[x,1] )
      ? files[x,1] + ' is a new file that needs to be added'
   endif
next

return NIL

// End of STRUCOMP.PRG
